﻿using System;
using System.Windows;
using Microsoft.Synchronization;
using Microsoft.Synchronization.Files;
using System.IO;
using System.Collections.Generic;

namespace SyncFiles
{
    public partial class MainWindow : Window
    {
        #region Private Data Members
        Guid sourceReplicaID;
        Guid destinationReplicaID;
        String sourcePath =String.Empty;
        String destinationPath = String.Empty;
        FileSyncProvider sourceProvider;
        FileSyncProvider destinationProvider;
        #endregion

        public MainWindow()
        {
            InitializeComponent();
        }

        private bool Synchronize(String source, String destination)
        {
            SyncOperationStatistics syncOperationStatistics;
            //Assign Unique Guids to the source and destination replicas
            sourceReplicaID = GetReplicaID(Path.Combine(source,"ReplicaID"));
            destinationReplicaID = GetReplicaID(Path.Combine(destination,"ReplicaID"));

            //Create the source Sync Provider, attach the path and assign the ReplicaID
            sourceProvider = new FileSyncProvider(sourceReplicaID, source);
            //Create the destination Sync Provider, attach the path and assign the ReplicaID
            destinationProvider = new FileSyncProvider(destinationReplicaID, destination);
            
            SyncOrchestrator synchronizationAgent = new SyncOrchestrator();
            synchronizationAgent.LocalProvider = sourceProvider;
            synchronizationAgent.RemoteProvider = destinationProvider;

            try
            {
                //Start the synchronization process
                syncOperationStatistics = synchronizationAgent.Synchronize();
                //Assign synchronization statistics to the lstStatistics control
                lstStatistics.Items.Add("Download Applied: "+ syncOperationStatistics.DownloadChangesApplied.ToString());
                lstStatistics.Items.Add("Download Failed: " + syncOperationStatistics.DownloadChangesFailed.ToString());
                lstStatistics.Items.Add("Download Total: " + syncOperationStatistics.DownloadChangesTotal.ToString());
                lstStatistics.Items.Add("Upload Total: " + syncOperationStatistics.UploadChangesApplied.ToString());
                lstStatistics.Items.Add("Upload Total: " + syncOperationStatistics.UploadChangesFailed.ToString());
                lstStatistics.Items.Add("Upload Total: " + syncOperationStatistics.UploadChangesTotal.ToString());                
            }
            catch (Microsoft.Synchronization.SyncException se)
            {
                MessageBox.Show(se.Message, "Sync Files - Error");
                return false;
            }
            finally
            {
                // Release resources once done
                if (sourceProvider != null) 
                    sourceProvider.Dispose();
                if (destinationProvider != null) 
                    destinationProvider.Dispose();
            }

            return true;
        }

        private void PopulateSourceFileList()
        {
            List<String> sourceFiles = GetFilesAndDirectories(sourcePath);
            lstSource.Items.Clear();

            foreach (String item in sourceFiles)
                lstSource.Items.Add(item);            
        }

        private void PopulateDestinationFileList()
        {
            List<String> destinationFiles = GetFilesAndDirectories(destinationPath);
            lstDestination.Items.Clear();

            foreach (String item in destinationFiles)
                lstDestination.Items.Add(item);
        }
        
        private void btnSyncFiles_Click(object sender, RoutedEventArgs e)
        {
            if (!String.IsNullOrEmpty(sourcePath) && !String.IsNullOrEmpty(destinationPath))
            {
                btnSyncFiles.IsEnabled = false; //Disable the button before synchronization starts
                bool returnValue = Synchronize(sourcePath, destinationPath);
                btnSyncFiles.IsEnabled = true; //Enable the button after synchronization is complete
                PopulateSourceFileList();
                PopulateDestinationFileList();

                if (false != returnValue)
                MessageBox.Show("Synchronization Finished...", "Working with Built-In Sync Providers"); 
            }
            else
            {
                MessageBox.Show("Please select appropriate source and destination folders to synchronize", "Sync Files - Error");
            }
        }
        
        private Guid GetReplicaID(string guidPath)
        {
            if (!File.Exists(guidPath)) //Create a new GUID and store it in a file
            {
                Guid replicaID = Guid.NewGuid();
                using (FileStream fileStream = File.Open(guidPath, FileMode.Create))
                {
                    using (StreamWriter streamWriter = new StreamWriter(fileStream))
                    {
                        streamWriter.WriteLine(replicaID.ToString());
                    }
                }
                return replicaID;
            }
            else //Read and return the GUID from the file
            {
                using (FileStream fileStream = File.Open(guidPath, FileMode.Open))
                {
                    using (StreamReader streamReader = new StreamReader(fileStream))
                    {
                        return new Guid(streamReader.ReadLine());
                    }
                }
            }
        }

        private void btnSource_Click(object sender, RoutedEventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.Title = "Select Source Folder";
            folderBrowserDialog.InitialExpandedFolder = @"C:\";
            
            if (folderBrowserDialog.ShowDialog(this) == true)
            {
                sourcePath = folderBrowserDialog.SelectedFolder;
                txtSource.Text = sourcePath;
                PopulateSourceFileList();
            }
        }

        private void btnDestination_Click(object sender, RoutedEventArgs e)
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.Title = "Select Destination Folder";
            folderBrowserDialog.InitialExpandedFolder = @"C:\";

            if (folderBrowserDialog.ShowDialog(this) == true)
            {
                destinationPath = folderBrowserDialog.SelectedFolder;
                txtDestination.Text = destinationPath;
                PopulateDestinationFileList();
            }
        }

        private List<string> GetFilesAndDirectories(String directory)
        {
            List<String> result = new List<String>();
            Stack<String> stack = new Stack<String>();
            stack.Push(directory);

            while (stack.Count > 0)
            {
                String temp = stack.Pop();
                try
                {
                    result.AddRange(Directory.GetFiles(temp, "*.*"));

                    foreach (string directoryName in Directory.GetDirectories(temp))
                    {
                        stack.Push(directoryName);
                    }
                }
                catch
                {
                    throw new Exception("Error retrieving file or directory.");
                }
            }

            return result;
         }

     }
}